﻿using Infragistics;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace Schedule_ScheduleViewAppBackground
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {

            var data = new ScheduleData();
            data.Resources.Add(new ResourceItem { Id = "0", Name = "Zero" });
            data.Resources.Add(new ResourceItem { Id = "1", Name = "One" });

            data.Calendars.Add(new ResourceCalendarItem { Id = "0", Name = "Main", OwningResourceId = "0" });
            data.Calendars.Add(new ResourceCalendarItem { Id = "1", Name = "Main", OwningResourceId = "1" });

            DateTime now = DateTime.Now.ToUniversalTime();
            data.Appointments.Add(new AppointmentItem { Id = "0", OwningResourceId = "0", OwningCalendarId = "0", Description = "This is when the app started", Start = now, End = now.AddMinutes(30), Subject = "Now", BorderColor = Colors.Red });
            data.Appointments.Add(new AppointmentItem { Id = "1", OwningResourceId = "0", OwningCalendarId = "0", Description = "This will happen later", Start = now.AddHours(1), End = now.AddHours(1).AddMinutes(30), Subject = "Later", BorderColor = Colors.Yellow });

            this.DataContext = data;

            InitializeComponent();
        }
    }

    public class ScheduleData
    {
        private ObservableCollection<ResourceItem> _resources;
        private ObservableCollection<ResourceCalendarItem> _calendars;
        private ObservableCollection<AppointmentItem> _appointments;

        public ScheduleData()
        {
            _resources = new ObservableCollection<ResourceItem>();
            _calendars = new ObservableCollection<ResourceCalendarItem>();
            _appointments = new ObservableCollection<AppointmentItem>();
        }

        public ObservableCollection<ResourceItem> Resources { get { return _resources; } }
        public ObservableCollection<ResourceCalendarItem> Calendars { get { return _calendars; } }
        public ObservableCollection<AppointmentItem> Appointments { get { return _appointments; } }
    }

    public class DataItemBase : PropertyChangeNotifier
    {
        protected bool SetField<T>(ref T member, T newValue, string propertyName)
        {
            if (EqualityComparer<T>.Default.Equals(member, newValue))
                return false;

            member = newValue;
            this.OnPropertyChanged(propertyName);
            return true;
        }
    }

    public class ResourceItem : DataItemBase
    {
        private string _id;
        public string Id
        {
            get { return _id; }
            set { this.SetField(ref _id, value, "Id"); }
        }

        private string _name;
        public string Name
        {
            get { return _name; }
            set { this.SetField(ref _name, value, "Name"); }
        }
    }

    public class ResourceCalendarItem : DataItemBase
    {
        private string _id;
        public string Id
        {
            get { return _id; }
            set { this.SetField(ref _id, value, "Id"); }
        }

        private string _name;
        public string Name
        {
            get { return _name; }
            set { this.SetField(ref _name, value, "Name"); }
        }

        private string _owningResourceOwningResourceId;
        public string OwningResourceId
        {
            get { return _owningResourceOwningResourceId; }
            set { this.SetField(ref _owningResourceOwningResourceId, value, "OwningResourceId"); }
        }
    }

    public class ActivityItem : DataItemBase
    {
        private string _id;
        public string Id
        {
            get { return _id; }
            set { this.SetField(ref _id, value, "Id"); }
        }

        private string _description;
        public string Description
        {
            get { return _description; }
            set { this.SetField(ref _description, value, "Description"); }
        }

        private DateTime _start;
        public DateTime Start
        {
            get { return _start; }
            set { this.SetField(ref _start, value, "Start"); }
        }

        private DateTime _end;
        public DateTime End
        {
            get { return _end; }
            set { this.SetField(ref _end, value, "End"); }
        }

        private string _owningResourceId;
        public string OwningResourceId
        {
            get { return _owningResourceId; }
            set { this.SetField(ref _owningResourceId, value, "OwningResourceId"); }
        }

        private string _owningCalendarId;
        public string OwningCalendarId
        {
            get { return _owningCalendarId; }
            set { this.SetField(ref _owningCalendarId, value, "OwningCalendarId"); }
        }

        private Color _borderColor = Colors.Black;
        public Color BorderColor
        {
            get { return _borderColor; }
            set { this.SetField(ref _borderColor, value, "BorderColor"); }
        }
    }

    public class AppointmentItem : ActivityItem
    {
        private string _subject;
        public string Subject
        {
            get { return _subject; }
            set { this.SetField(ref _subject, value, "Subject"); }
        }
    }
}
